/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.task.field;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import nl.cloudfarming.client.model.EntitySavable;
import nl.cloudfarming.client.task.field.tree.FieldTaskNode;
import org.netbeans.api.actions.Savable;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;

public class FieldTaskDataObject extends MultiDataObject implements PropertyChangeListener {

    private static final Logger LOGGER = Logger.getLogger(FieldTaskDataObject.class.getCanonicalName());
    private InstanceContent ic;
    private AbstractLookup intern;
    private FieldTask fieldTask;
    private FieldTaskManager fieldTaskManager;

    public FieldTaskDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        registerEditor(FieldTask.MIME_TYPE, true);

        ic = new InstanceContent();
        intern = new AbstractLookup(this.ic);

        fieldTaskManager = Lookup.getDefault().lookup(FieldTaskManager.class);
        assert fieldTaskManager != null : "No FieldTaskManager class found in the default lookup, unable to load the DataObject";

        fieldTask = fieldTaskManager.loadTask(getPrimaryFile());
        assert fieldTask != null;
        assert fieldTask.getLookup() != null : "FieldTask lookup is null, not properly deserialized";
        LOGGER.log(Level.FINE, "FieldTaskDataObject loaded {0}", System.identityHashCode(fieldTask));
        ic.add(fieldTask);
        ic.add(new SimpleLayer(fieldTask.getName()));
        fieldTask.addPropertyChangeListener(this);
    }

    @Override
    protected int associateLookup() {
        return 1;
    }

    @Override
    protected Node createNodeDelegate() {
        return new FieldTaskNode(super.createNodeDelegate(), fieldTask);
    }

    @Override
    public Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), fieldTask.getLookup(), intern);
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        Savable existingSavable = getLookup().lookup(Savable.class);
        if (existingSavable == null) {
            ic.add(new FieldTaskSavable());
        }
    }

    @Override
    protected void handleDelete() throws IOException {
        super.handleDelete();
        FieldTaskSavable existingSavable = getLookup().lookup(FieldTaskSavable.class);
        if (existingSavable != null) {
            existingSavable.destroy();
        }
    }

    private  static final Icon ICON = ImageUtilities.loadImageIcon("nl/cloudfarming/client/task/field/to_do_list_checked_1.png", true);

    private class FieldTaskSavable extends EntitySavable {

        public FieldTaskSavable() {
            super(fieldTask, ICON);
            register();
        }

        @Override
        protected String findDisplayName() {
            return fieldTask.getName();
        }

        @Override
        protected void handleSave() throws IOException {
            getDataObject().fieldTaskManager.save(fieldTask);
            destroy();
        }

        @Override
        public void destroy() {
            getDataObject().ic.remove(FieldTaskSavable.this);
            unregister();
        }

        @Override
        public void dispose() {
            getDataObject().dispose();
        }

        @Override
        public boolean equals(Object obj) {
            if (obj instanceof FieldTaskSavable) {
                FieldTaskSavable savable = (FieldTaskSavable) obj;
                return getDataObject() == savable.getDataObject();
            }
            return false;
        }

        private FieldTaskDataObject getDataObject() {
            return FieldTaskDataObject.this;
        }

        @Override
        public int hashCode() {
            return getDataObject().hashCode();
        }
    }
}
